from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.finding_analysis_state import FindingAnalysisState
from ..types import UNSET, Unset

T = TypeVar("T", bound="FindingAnalysis")


@_attrs_define
class FindingAnalysis:
    """
    Attributes:
        state (Union[Unset, FindingAnalysisState]):
        is_suppressed (Union[Unset, bool]):
    """

    state: Union[Unset, FindingAnalysisState] = UNSET
    is_suppressed: Union[Unset, bool] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        state: Union[Unset, str] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state.value

        is_suppressed = self.is_suppressed

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if state is not UNSET:
            field_dict["state"] = state
        if is_suppressed is not UNSET:
            field_dict["isSuppressed"] = is_suppressed

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _state = d.pop("state", UNSET)
        state: Union[Unset, FindingAnalysisState]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = FindingAnalysisState(_state)

        is_suppressed = d.pop("isSuppressed", UNSET)

        finding_analysis = cls(
            state=state,
            is_suppressed=is_suppressed,
        )

        finding_analysis.additional_properties = d
        return finding_analysis

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import pytest

import owasp_dt
import test
from owasp_dt.api.metrics import get_project_current_metrics
from owasp_dt.api.project import get_projects


@pytest.mark.depends(on=['test/test_upload.py::test_upload_sbom'])
def test_search_project_by_name(client: owasp_dt.Client):
    resp = get_projects.sync_detailed(client=client, name=test.project_name)
    projects = resp.parsed
    assert len(projects) > 0
    assert projects[0].uuid is not None
    test.project_uuid = projects[0].uuid

@pytest.mark.depends(on=['test/test_upload.py::test_get_scan_status', 'test_search_project_by_name'])
def test_get_project_metrics(client: owasp_dt.Client):
    resp = get_project_current_metrics.sync_detailed(client=client, uuid=test.project_uuid)
    metrics = resp.parsed



from enum import Enum


class NotificationRuleNotificationLevel(str, Enum):
    ERROR = "ERROR"
    INFORMATIONAL = "INFORMATIONAL"
    WARNING = "WARNING"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="TagListResponseItem")


@_attrs_define
class TagListResponseItem:
    """
    Attributes:
        name (str): Name of the tag
        project_count (int): Number of projects assigned to this tag
        collection_project_count (int): Number of collection projects assigned to this tag
        policy_count (int): Number of policies assigned to this tag
        notification_rule_count (int): Number of notification rules assigned to this tag
    """

    name: str
    project_count: int
    collection_project_count: int
    policy_count: int
    notification_rule_count: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        project_count = self.project_count

        collection_project_count = self.collection_project_count

        policy_count = self.policy_count

        notification_rule_count = self.notification_rule_count

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "projectCount": project_count,
                "collectionProjectCount": collection_project_count,
                "policyCount": policy_count,
                "notificationRuleCount": notification_rule_count,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        project_count = d.pop("projectCount")

        collection_project_count = d.pop("collectionProjectCount")

        policy_count = d.pop("policyCount")

        notification_rule_count = d.pop("notificationRuleCount")

        tag_list_response_item = cls(
            name=name,
            project_count=project_count,
            collection_project_count=collection_project_count,
            policy_count=policy_count,
            notification_rule_count=notification_rule_count,
        )

        tag_list_response_item.additional_properties = d
        return tag_list_response_item

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="VulnerabilityMetrics")


@_attrs_define
class VulnerabilityMetrics:
    """
    Attributes:
        measured_at (int): UNIX epoch timestamp in milliseconds
        year (Union[Unset, int]):
        month (Union[Unset, int]):
        count (Union[Unset, int]):
    """

    measured_at: int
    year: Union[Unset, int] = UNSET
    month: Union[Unset, int] = UNSET
    count: Union[Unset, int] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        measured_at = self.measured_at

        year = self.year

        month = self.month

        count = self.count

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "measuredAt": measured_at,
            }
        )
        if year is not UNSET:
            field_dict["year"] = year
        if month is not UNSET:
            field_dict["month"] = month
        if count is not UNSET:
            field_dict["count"] = count

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        measured_at = d.pop("measuredAt")

        year = d.pop("year", UNSET)

        month = d.pop("month", UNSET)

        count = d.pop("count", UNSET)

        vulnerability_metrics = cls(
            measured_at=measured_at,
            year=year,
            month=month,
            count=count,
        )

        vulnerability_metrics.additional_properties = d
        return vulnerability_metrics

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class RepositoryType(str, Enum):
    CARGO = "CARGO"
    COMPOSER = "COMPOSER"
    CPAN = "CPAN"
    GEM = "GEM"
    GITHUB = "GITHUB"
    GO_MODULES = "GO_MODULES"
    HACKAGE = "HACKAGE"
    HEX = "HEX"
    MAVEN = "MAVEN"
    NIXPKGS = "NIXPKGS"
    NPM = "NPM"
    NUGET = "NUGET"
    PYPI = "PYPI"
    UNSUPPORTED = "UNSUPPORTED"

    def __str__(self) -> str:
        return str(self.value)



from pathlib import Path
from time import sleep

import pytest

import owasp_dt
import test
from owasp_dt.api.bom import upload_bom
from owasp_dt.api.event import is_token_being_processed_1
from owasp_dt.models import UploadBomBody, IsTokenBeingProcessedResponse

def test_upload_sbom(client: owasp_dt.Client):
    with open(test.base_dir / "files/test.sbom.xml") as sbom_file:
        resp = upload_bom.sync_detailed(client=client, body=UploadBomBody(
            project_name=test.project_name,
            auto_create=True,
            bom=sbom_file.read()
        ))
        upload = resp.parsed
        assert upload is not None, "API call failed. Check client permissions."
        assert upload.token is not None
        test.upload_token = upload.token


@pytest.mark.depends(on=['test_upload_sbom'])
def test_get_scan_status(client: owasp_dt.Client):
    max_tries = 10
    i = 0
    for i in range(max_tries):
        resp = is_token_being_processed_1.sync_detailed(client=client, uuid=test.upload_token)
        status = resp.parsed
        assert isinstance(status, IsTokenBeingProcessedResponse)
        if not status.processing:
            break
        sleep(1)

    assert i < max_tries, f"Scan not finished within {max_tries} seconds"



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="UploadVex1Body")


@_attrs_define
class UploadVex1Body:
    """
    Attributes:
        project (Union[Unset, str]):
        project_name (Union[Unset, str]):
        project_version (Union[Unset, str]):
        vex (Union[Unset, str]):
    """

    project: Union[Unset, str] = UNSET
    project_name: Union[Unset, str] = UNSET
    project_version: Union[Unset, str] = UNSET
    vex: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        project = self.project

        project_name = self.project_name

        project_version = self.project_version

        vex = self.vex

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if project is not UNSET:
            field_dict["project"] = project
        if project_name is not UNSET:
            field_dict["projectName"] = project_name
        if project_version is not UNSET:
            field_dict["projectVersion"] = project_version
        if vex is not UNSET:
            field_dict["vex"] = vex

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.project, Unset):
            files.append(("project", (None, str(self.project).encode(), "text/plain")))

        if not isinstance(self.project_name, Unset):
            files.append(
                ("projectName", (None, str(self.project_name).encode(), "text/plain"))
            )

        if not isinstance(self.project_version, Unset):
            files.append(
                (
                    "projectVersion",
                    (None, str(self.project_version).encode(), "text/plain"),
                )
            )

        if not isinstance(self.vex, Unset):
            files.append(("vex", (None, str(self.vex).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        project = d.pop("project", UNSET)

        project_name = d.pop("projectName", UNSET)

        project_version = d.pop("projectVersion", UNSET)

        vex = d.pop("vex", UNSET)

        upload_vex_1_body = cls(
            project=project,
            project_name=project_name,
            project_version=project_version,
            vex=vex,
        )

        upload_vex_1_body.additional_properties = d
        return upload_vex_1_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.vulnerability_severity import VulnerabilitySeverity
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.affected_component import AffectedComponent
    from ..models.component import Component
    from ..models.cwe import Cwe
    from ..models.finding_attrib import FindingAttrib
    from ..models.service_component import ServiceComponent
    from ..models.vulnerability_alias import VulnerabilityAlias


T = TypeVar("T", bound="Vulnerability")


@_attrs_define
class Vulnerability:
    """
    Attributes:
        vuln_id (str):
        source (str):
        uuid (UUID):
        friendly_vuln_id (Union[Unset, str]):
        title (Union[Unset, str]):
        sub_title (Union[Unset, str]):
        description (Union[Unset, str]):
        detail (Union[Unset, str]):
        recommendation (Union[Unset, str]):
        references (Union[Unset, str]):
        credits_ (Union[Unset, str]):
        created (Union[Unset, datetime.datetime]):
        published (Union[Unset, datetime.datetime]):
        updated (Union[Unset, datetime.datetime]):
        cwes (Union[Unset, list['Cwe']]):
        cvss_v2_base_score (Union[Unset, float]):
        cvss_v2_impact_sub_score (Union[Unset, float]):
        cvss_v2_exploitability_sub_score (Union[Unset, float]):
        cvss_v2_vector (Union[Unset, str]):
        cvss_v3_base_score (Union[Unset, float]):
        cvss_v3_impact_sub_score (Union[Unset, float]):
        cvss_v3_exploitability_sub_score (Union[Unset, float]):
        cvss_v3_vector (Union[Unset, str]):
        owasp_rr_likelihood_score (Union[Unset, float]):
        owasp_rr_technical_impact_score (Union[Unset, float]):
        owasp_rr_business_impact_score (Union[Unset, float]):
        owasp_rr_vector (Union[Unset, str]):
        severity (Union[Unset, VulnerabilitySeverity]):
        vulnerable_versions (Union[Unset, str]):
        patched_versions (Union[Unset, str]):
        epss_score (Union[Unset, float]):
        epss_percentile (Union[Unset, float]):
        components (Union[Unset, list['Component']]):
        service_components (Union[Unset, list['ServiceComponent']]):
        aliases (Union[Unset, list['VulnerabilityAlias']]):
        finding_attribution (Union[Unset, FindingAttrib]):
        affected_project_count (Union[Unset, int]):
        affected_active_project_count (Union[Unset, int]):
        affected_inactive_project_count (Union[Unset, int]):
        affected_components (Union[Unset, list['AffectedComponent']]):
        cwe (Union[Unset, Cwe]):
    """

    vuln_id: str
    source: str
    uuid: UUID
    friendly_vuln_id: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    sub_title: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    detail: Union[Unset, str] = UNSET
    recommendation: Union[Unset, str] = UNSET
    references: Union[Unset, str] = UNSET
    credits_: Union[Unset, str] = UNSET
    created: Union[Unset, datetime.datetime] = UNSET
    published: Union[Unset, datetime.datetime] = UNSET
    updated: Union[Unset, datetime.datetime] = UNSET
    cwes: Union[Unset, list["Cwe"]] = UNSET
    cvss_v2_base_score: Union[Unset, float] = UNSET
    cvss_v2_impact_sub_score: Union[Unset, float] = UNSET
    cvss_v2_exploitability_sub_score: Union[Unset, float] = UNSET
    cvss_v2_vector: Union[Unset, str] = UNSET
    cvss_v3_base_score: Union[Unset, float] = UNSET
    cvss_v3_impact_sub_score: Union[Unset, float] = UNSET
    cvss_v3_exploitability_sub_score: Union[Unset, float] = UNSET
    cvss_v3_vector: Union[Unset, str] = UNSET
    owasp_rr_likelihood_score: Union[Unset, float] = UNSET
    owasp_rr_technical_impact_score: Union[Unset, float] = UNSET
    owasp_rr_business_impact_score: Union[Unset, float] = UNSET
    owasp_rr_vector: Union[Unset, str] = UNSET
    severity: Union[Unset, VulnerabilitySeverity] = UNSET
    vulnerable_versions: Union[Unset, str] = UNSET
    patched_versions: Union[Unset, str] = UNSET
    epss_score: Union[Unset, float] = UNSET
    epss_percentile: Union[Unset, float] = UNSET
    components: Union[Unset, list["Component"]] = UNSET
    service_components: Union[Unset, list["ServiceComponent"]] = UNSET
    aliases: Union[Unset, list["VulnerabilityAlias"]] = UNSET
    finding_attribution: Union[Unset, "FindingAttrib"] = UNSET
    affected_project_count: Union[Unset, int] = UNSET
    affected_active_project_count: Union[Unset, int] = UNSET
    affected_inactive_project_count: Union[Unset, int] = UNSET
    affected_components: Union[Unset, list["AffectedComponent"]] = UNSET
    cwe: Union[Unset, "Cwe"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        vuln_id = self.vuln_id

        source = self.source

        uuid = str(self.uuid)

        friendly_vuln_id = self.friendly_vuln_id

        title = self.title

        sub_title = self.sub_title

        description = self.description

        detail = self.detail

        recommendation = self.recommendation

        references = self.references

        credits_ = self.credits_

        created: Union[Unset, str] = UNSET
        if not isinstance(self.created, Unset):
            created = self.created.isoformat()

        published: Union[Unset, str] = UNSET
        if not isinstance(self.published, Unset):
            published = self.published.isoformat()

        updated: Union[Unset, str] = UNSET
        if not isinstance(self.updated, Unset):
            updated = self.updated.isoformat()

        cwes: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.cwes, Unset):
            cwes = []
            for cwes_item_data in self.cwes:
                cwes_item = cwes_item_data.to_dict()
                cwes.append(cwes_item)

        cvss_v2_base_score = self.cvss_v2_base_score

        cvss_v2_impact_sub_score = self.cvss_v2_impact_sub_score

        cvss_v2_exploitability_sub_score = self.cvss_v2_exploitability_sub_score

        cvss_v2_vector = self.cvss_v2_vector

        cvss_v3_base_score = self.cvss_v3_base_score

        cvss_v3_impact_sub_score = self.cvss_v3_impact_sub_score

        cvss_v3_exploitability_sub_score = self.cvss_v3_exploitability_sub_score

        cvss_v3_vector = self.cvss_v3_vector

        owasp_rr_likelihood_score = self.owasp_rr_likelihood_score

        owasp_rr_technical_impact_score = self.owasp_rr_technical_impact_score

        owasp_rr_business_impact_score = self.owasp_rr_business_impact_score

        owasp_rr_vector = self.owasp_rr_vector

        severity: Union[Unset, str] = UNSET
        if not isinstance(self.severity, Unset):
            severity = self.severity.value

        vulnerable_versions = self.vulnerable_versions

        patched_versions = self.patched_versions

        epss_score = self.epss_score

        epss_percentile = self.epss_percentile

        components: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.components, Unset):
            components = []
            for components_item_data in self.components:
                components_item = components_item_data.to_dict()
                components.append(components_item)

        service_components: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.service_components, Unset):
            service_components = []
            for service_components_item_data in self.service_components:
                service_components_item = service_components_item_data.to_dict()
                service_components.append(service_components_item)

        aliases: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.aliases, Unset):
            aliases = []
            for aliases_item_data in self.aliases:
                aliases_item = aliases_item_data.to_dict()
                aliases.append(aliases_item)

        finding_attribution: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.finding_attribution, Unset):
            finding_attribution = self.finding_attribution.to_dict()

        affected_project_count = self.affected_project_count

        affected_active_project_count = self.affected_active_project_count

        affected_inactive_project_count = self.affected_inactive_project_count

        affected_components: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.affected_components, Unset):
            affected_components = []
            for affected_components_item_data in self.affected_components:
                affected_components_item = affected_components_item_data.to_dict()
                affected_components.append(affected_components_item)

        cwe: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.cwe, Unset):
            cwe = self.cwe.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "vulnId": vuln_id,
                "source": source,
                "uuid": uuid,
            }
        )
        if friendly_vuln_id is not UNSET:
            field_dict["friendlyVulnId"] = friendly_vuln_id
        if title is not UNSET:
            field_dict["title"] = title
        if sub_title is not UNSET:
            field_dict["subTitle"] = sub_title
        if description is not UNSET:
            field_dict["description"] = description
        if detail is not UNSET:
            field_dict["detail"] = detail
        if recommendation is not UNSET:
            field_dict["recommendation"] = recommendation
        if references is not UNSET:
            field_dict["references"] = references
        if credits_ is not UNSET:
            field_dict["credits"] = credits_
        if created is not UNSET:
            field_dict["created"] = created
        if published is not UNSET:
            field_dict["published"] = published
        if updated is not UNSET:
            field_dict["updated"] = updated
        if cwes is not UNSET:
            field_dict["cwes"] = cwes
        if cvss_v2_base_score is not UNSET:
            field_dict["cvssV2BaseScore"] = cvss_v2_base_score
        if cvss_v2_impact_sub_score is not UNSET:
            field_dict["cvssV2ImpactSubScore"] = cvss_v2_impact_sub_score
        if cvss_v2_exploitability_sub_score is not UNSET:
            field_dict["cvssV2ExploitabilitySubScore"] = (
                cvss_v2_exploitability_sub_score
            )
        if cvss_v2_vector is not UNSET:
            field_dict["cvssV2Vector"] = cvss_v2_vector
        if cvss_v3_base_score is not UNSET:
            field_dict["cvssV3BaseScore"] = cvss_v3_base_score
        if cvss_v3_impact_sub_score is not UNSET:
            field_dict["cvssV3ImpactSubScore"] = cvss_v3_impact_sub_score
        if cvss_v3_exploitability_sub_score is not UNSET:
            field_dict["cvssV3ExploitabilitySubScore"] = (
                cvss_v3_exploitability_sub_score
            )
        if cvss_v3_vector is not UNSET:
            field_dict["cvssV3Vector"] = cvss_v3_vector
        if owasp_rr_likelihood_score is not UNSET:
            field_dict["owaspRRLikelihoodScore"] = owasp_rr_likelihood_score
        if owasp_rr_technical_impact_score is not UNSET:
            field_dict["owaspRRTechnicalImpactScore"] = owasp_rr_technical_impact_score
        if owasp_rr_business_impact_score is not UNSET:
            field_dict["owaspRRBusinessImpactScore"] = owasp_rr_business_impact_score
        if owasp_rr_vector is not UNSET:
            field_dict["owaspRRVector"] = owasp_rr_vector
        if severity is not UNSET:
            field_dict["severity"] = severity
        if vulnerable_versions is not UNSET:
            field_dict["vulnerableVersions"] = vulnerable_versions
        if patched_versions is not UNSET:
            field_dict["patchedVersions"] = patched_versions
        if epss_score is not UNSET:
            field_dict["epssScore"] = epss_score
        if epss_percentile is not UNSET:
            field_dict["epssPercentile"] = epss_percentile
        if components is not UNSET:
            field_dict["components"] = components
        if service_components is not UNSET:
            field_dict["serviceComponents"] = service_components
        if aliases is not UNSET:
            field_dict["aliases"] = aliases
        if finding_attribution is not UNSET:
            field_dict["findingAttribution"] = finding_attribution
        if affected_project_count is not UNSET:
            field_dict["affectedProjectCount"] = affected_project_count
        if affected_active_project_count is not UNSET:
            field_dict["affectedActiveProjectCount"] = affected_active_project_count
        if affected_inactive_project_count is not UNSET:
            field_dict["affectedInactiveProjectCount"] = affected_inactive_project_count
        if affected_components is not UNSET:
            field_dict["affectedComponents"] = affected_components
        if cwe is not UNSET:
            field_dict["cwe"] = cwe

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.affected_component import AffectedComponent
        from ..models.component import Component
        from ..models.cwe import Cwe
        from ..models.finding_attrib import FindingAttrib
        from ..models.service_component import ServiceComponent
        from ..models.vulnerability_alias import VulnerabilityAlias

        d = dict(src_dict)
        vuln_id = d.pop("vulnId")

        source = d.pop("source")

        uuid = UUID(d.pop("uuid"))

        friendly_vuln_id = d.pop("friendlyVulnId", UNSET)

        title = d.pop("title", UNSET)

        sub_title = d.pop("subTitle", UNSET)

        description = d.pop("description", UNSET)

        detail = d.pop("detail", UNSET)

        recommendation = d.pop("recommendation", UNSET)

        references = d.pop("references", UNSET)

        credits_ = d.pop("credits", UNSET)

        _created = d.pop("created", UNSET)
        created: Union[Unset, datetime.datetime]
        if isinstance(_created, Unset):
            created = UNSET
        else:
            created = isoparse(_created)

        _published = d.pop("published", UNSET)
        published: Union[Unset, datetime.datetime]
        if isinstance(_published, Unset):
            published = UNSET
        else:
            published = isoparse(_published)

        _updated = d.pop("updated", UNSET)
        updated: Union[Unset, datetime.datetime]
        if isinstance(_updated, Unset):
            updated = UNSET
        else:
            updated = isoparse(_updated)

        cwes = []
        _cwes = d.pop("cwes", UNSET)
        for cwes_item_data in _cwes or []:
            cwes_item = Cwe.from_dict(cwes_item_data)

            cwes.append(cwes_item)

        cvss_v2_base_score = d.pop("cvssV2BaseScore", UNSET)

        cvss_v2_impact_sub_score = d.pop("cvssV2ImpactSubScore", UNSET)

        cvss_v2_exploitability_sub_score = d.pop("cvssV2ExploitabilitySubScore", UNSET)

        cvss_v2_vector = d.pop("cvssV2Vector", UNSET)

        cvss_v3_base_score = d.pop("cvssV3BaseScore", UNSET)

        cvss_v3_impact_sub_score = d.pop("cvssV3ImpactSubScore", UNSET)

        cvss_v3_exploitability_sub_score = d.pop("cvssV3ExploitabilitySubScore", UNSET)

        cvss_v3_vector = d.pop("cvssV3Vector", UNSET)

        owasp_rr_likelihood_score = d.pop("owaspRRLikelihoodScore", UNSET)

        owasp_rr_technical_impact_score = d.pop("owaspRRTechnicalImpactScore", UNSET)

        owasp_rr_business_impact_score = d.pop("owaspRRBusinessImpactScore", UNSET)

        owasp_rr_vector = d.pop("owaspRRVector", UNSET)

        _severity = d.pop("severity", UNSET)
        severity: Union[Unset, VulnerabilitySeverity]
        if isinstance(_severity, Unset):
            severity = UNSET
        else:
            severity = VulnerabilitySeverity(_severity)

        vulnerable_versions = d.pop("vulnerableVersions", UNSET)

        patched_versions = d.pop("patchedVersions", UNSET)

        epss_score = d.pop("epssScore", UNSET)

        epss_percentile = d.pop("epssPercentile", UNSET)

        components = []
        _components = d.pop("components", UNSET)
        for components_item_data in _components or []:
            components_item = Component.from_dict(components_item_data)

            components.append(components_item)

        service_components = []
        _service_components = d.pop("serviceComponents", UNSET)
        for service_components_item_data in _service_components or []:
            service_components_item = ServiceComponent.from_dict(
                service_components_item_data
            )

            service_components.append(service_components_item)

        aliases = []
        _aliases = d.pop("aliases", UNSET)
        for aliases_item_data in _aliases or []:
            aliases_item = VulnerabilityAlias.from_dict(aliases_item_data)

            aliases.append(aliases_item)

        _finding_attribution = d.pop("findingAttribution", UNSET)
        finding_attribution: Union[Unset, FindingAttrib]
        if isinstance(_finding_attribution, Unset):
            finding_attribution = UNSET
        else:
            finding_attribution = FindingAttrib.from_dict(_finding_attribution)

        affected_project_count = d.pop("affectedProjectCount", UNSET)

        affected_active_project_count = d.pop("affectedActiveProjectCount", UNSET)

        affected_inactive_project_count = d.pop("affectedInactiveProjectCount", UNSET)

        affected_components = []
        _affected_components = d.pop("affectedComponents", UNSET)
        for affected_components_item_data in _affected_components or []:
            affected_components_item = AffectedComponent.from_dict(
                affected_components_item_data
            )

            affected_components.append(affected_components_item)

        _cwe = d.pop("cwe", UNSET)
        cwe: Union[Unset, Cwe]
        if isinstance(_cwe, Unset):
            cwe = UNSET
        else:
            cwe = Cwe.from_dict(_cwe)

        vulnerability = cls(
            vuln_id=vuln_id,
            source=source,
            uuid=uuid,
            friendly_vuln_id=friendly_vuln_id,
            title=title,
            sub_title=sub_title,
            description=description,
            detail=detail,
            recommendation=recommendation,
            references=references,
            credits_=credits_,
            created=created,
            published=published,
            updated=updated,
            cwes=cwes,
            cvss_v2_base_score=cvss_v2_base_score,
            cvss_v2_impact_sub_score=cvss_v2_impact_sub_score,
            cvss_v2_exploitability_sub_score=cvss_v2_exploitability_sub_score,
            cvss_v2_vector=cvss_v2_vector,
            cvss_v3_base_score=cvss_v3_base_score,
            cvss_v3_impact_sub_score=cvss_v3_impact_sub_score,
            cvss_v3_exploitability_sub_score=cvss_v3_exploitability_sub_score,
            cvss_v3_vector=cvss_v3_vector,
            owasp_rr_likelihood_score=owasp_rr_likelihood_score,
            owasp_rr_technical_impact_score=owasp_rr_technical_impact_score,
            owasp_rr_business_impact_score=owasp_rr_business_impact_score,
            owasp_rr_vector=owasp_rr_vector,
            severity=severity,
            vulnerable_versions=vulnerable_versions,
            patched_versions=patched_versions,
            epss_score=epss_score,
            epss_percentile=epss_percentile,
            components=components,
            service_components=service_components,
            aliases=aliases,
            finding_attribution=finding_attribution,
            affected_project_count=affected_project_count,
            affected_active_project_count=affected_active_project_count,
            affected_inactive_project_count=affected_inactive_project_count,
            affected_components=affected_components,
            cwe=cwe,
        )

        vulnerability.additional_properties = d
        return vulnerability

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .about import About
from .about_provider_data import AboutProviderData
from .about_provider_data_additional_property import AboutProviderDataAdditionalProperty
from .acl_mapping_request import AclMappingRequest
from .affected_component import AffectedComponent
from .affected_component_identity_type import AffectedComponentIdentityType
from .affected_component_version_type import AffectedComponentVersionType
from .affected_version_attribution import AffectedVersionAttribution
from .affected_version_attribution_source import AffectedVersionAttributionSource
from .analysis import Analysis
from .analysis_analysis_justification import AnalysisAnalysisJustification
from .analysis_analysis_response import AnalysisAnalysisResponse
from .analysis_analysis_state import AnalysisAnalysisState
from .analysis_comment import AnalysisComment
from .analysis_request import AnalysisRequest
from .analysis_request_analysis_justification import (
    AnalysisRequestAnalysisJustification,
)
from .analysis_request_analysis_response import AnalysisRequestAnalysisResponse
from .analysis_request_analysis_state import AnalysisRequestAnalysisState
from .api_key import ApiKey
from .bom_submit_request import BomSubmitRequest
from .bom_upload_response import BomUploadResponse
from .clone_project_request import CloneProjectRequest
from .component import Component
from .component_classifier import ComponentClassifier
from .component_property import ComponentProperty
from .component_property_property_type import ComponentPropertyPropertyType
from .config_property import ConfigProperty
from .config_property_property_type import ConfigPropertyPropertyType
from .create_scheduled_notification_rule_request import (
    CreateScheduledNotificationRuleRequest,
)
from .create_scheduled_notification_rule_request_notification_level import (
    CreateScheduledNotificationRuleRequestNotificationLevel,
)
from .create_scheduled_notification_rule_request_scope import (
    CreateScheduledNotificationRuleRequestScope,
)
from .cwe import Cwe
from .data_classification import DataClassification
from .data_classification_direction import DataClassificationDirection
from .dependency_graph_response import DependencyGraphResponse
from .dependency_metrics import DependencyMetrics
from .external_reference import ExternalReference
from .external_reference_type import ExternalReferenceType
from .finding import Finding
from .finding_analysis import FindingAnalysis
from .finding_analysis_state import FindingAnalysisState
from .finding_attrib import FindingAttrib
from .finding_attrib_analyzer_identity import FindingAttribAnalyzerIdentity
from .finding_component import FindingComponent
from .finding_vulnerability import FindingVulnerability
from .force_change_password_body import ForceChangePasswordBody
from .framework import Framework
from .get_all_components_sort_order import GetAllComponentsSortOrder
from .get_all_notification_rules_sort_order import GetAllNotificationRulesSortOrder
from .get_all_notification_rules_trigger_type import GetAllNotificationRulesTriggerType
from .get_all_permissions_response_200 import GetAllPermissionsResponse200
from .get_all_services_sort_order import GetAllServicesSortOrder
from .get_all_tags_sort_order import GetAllTagsSortOrder
from .get_all_vulnerabilities_sort_order import GetAllVulnerabilitiesSortOrder
from .get_children_projects_by_classifier_sort_order import (
    GetChildrenProjectsByClassifierSortOrder,
)
from .get_children_projects_by_tag_sort_order import GetChildrenProjectsByTagSortOrder
from .get_children_projects_sort_order import GetChildrenProjectsSortOrder
from .get_component_by_hash_sort_order import GetComponentByHashSortOrder
from .get_component_by_identity_sort_order import GetComponentByIdentitySortOrder
from .get_cwes_sort_order import GetCwesSortOrder
from .get_dependency_graph_for_component_response_200 import (
    GetDependencyGraphForComponentResponse200,
)
from .get_findings_by_project_source import GetFindingsByProjectSource
from .get_license_groups_sort_order import GetLicenseGroupsSortOrder
from .get_licenses_sort_order import GetLicensesSortOrder
from .get_policies_sort_order import GetPoliciesSortOrder
from .get_projects_by_classifier_sort_order import GetProjectsByClassifierSortOrder
from .get_projects_by_tag_sort_order import GetProjectsByTagSortOrder
from .get_projects_sort_order import GetProjectsSortOrder
from .get_projects_without_descendants_of_sort_order import (
    GetProjectsWithoutDescendantsOfSortOrder,
)
from .get_repositories_by_type_sort_order import GetRepositoriesByTypeSortOrder
from .get_repositories_by_type_type import GetRepositoriesByTypeType
from .get_repositories_sort_order import GetRepositoriesSortOrder
from .get_tagged_collection_projects_sort_order import (
    GetTaggedCollectionProjectsSortOrder,
)
from .get_tagged_notification_rules_sort_order import (
    GetTaggedNotificationRulesSortOrder,
)
from .get_tagged_policies_sort_order import GetTaggedPoliciesSortOrder
from .get_tagged_projects_sort_order import GetTaggedProjectsSortOrder
from .get_tags_for_policy_sort_order import GetTagsForPolicySortOrder
from .get_tags_sort_order import GetTagsSortOrder
from .get_violations_by_component_sort_order import GetViolationsByComponentSortOrder
from .get_violations_by_project_sort_order import GetViolationsByProjectSortOrder
from .get_violations_sort_order import GetViolationsSortOrder
from .get_vulnerabilities_by_component_sort_order import (
    GetVulnerabilitiesByComponentSortOrder,
)
from .identifiable_object import IdentifiableObject
from .invalid_bom_problem_details import InvalidBomProblemDetails
from .is_token_being_processed_response import IsTokenBeingProcessedResponse
from .ldap_user import LdapUser
from .license_ import License
from .license_group import LicenseGroup
from .managed_user import ManagedUser
from .mapped_ldap_group import MappedLdapGroup
from .mapped_ldap_group_request import MappedLdapGroupRequest
from .mapped_oidc_group import MappedOidcGroup
from .mapped_oidc_group_request import MappedOidcGroupRequest
from .notification_publisher import NotificationPublisher
from .notification_rule import NotificationRule
from .notification_rule_notification_level import NotificationRuleNotificationLevel
from .notification_rule_notify_on_item import NotificationRuleNotifyOnItem
from .notification_rule_scope import NotificationRuleScope
from .notification_rule_trigger_type import NotificationRuleTriggerType
from .oidc_group import OidcGroup
from .oidc_user import OidcUser
from .organizational_contact import OrganizationalContact
from .organizational_entity import OrganizationalEntity
from .permission import Permission
from .policy import Policy
from .policy_condition import PolicyCondition
from .policy_condition_operator import PolicyConditionOperator
from .policy_condition_subject import PolicyConditionSubject
from .policy_operator import PolicyOperator
from .policy_violation import PolicyViolation
from .policy_violation_state import PolicyViolationState
from .policy_violation_type import PolicyViolationType
from .portfolio_metrics import PortfolioMetrics
from .problem_details import ProblemDetails
from .project import Project
from .project_classifier import ProjectClassifier
from .project_collection_logic import ProjectCollectionLogic
from .project_metadata import ProjectMetadata
from .project_metrics import ProjectMetrics
from .project_metrics_collection_logic import ProjectMetricsCollectionLogic
from .project_operation_problem_details import ProjectOperationProblemDetails
from .project_operation_problem_details_errors import (
    ProjectOperationProblemDetailsErrors,
)
from .project_property import ProjectProperty
from .project_property_property_type import ProjectPropertyPropertyType
from .project_version import ProjectVersion
from .publisher import Publisher
from .repository import Repository
from .repository_meta_component import RepositoryMetaComponent
from .repository_meta_component_repository_type import (
    RepositoryMetaComponentRepositoryType,
)
from .repository_type import RepositoryType
from .retrieve_projects_sort_order import RetrieveProjectsSortOrder
from .score import Score
from .score_business_impact import ScoreBusinessImpact
from .score_likelihood import ScoreLikelihood
from .score_technical_impact import ScoreTechnicalImpact
from .search_result import SearchResult
from .search_result_results import SearchResultResults
from .search_result_results_additional_property_item import (
    SearchResultResultsAdditionalPropertyItem,
)
from .service_component import ServiceComponent
from .tag import Tag
from .tag_list_response_item import TagListResponseItem
from .tag_operation_problem_details import TagOperationProblemDetails
from .tag_operation_problem_details_errors import TagOperationProblemDetailsErrors
from .tagged_collection_project_list_response_item import (
    TaggedCollectionProjectListResponseItem,
)
from .tagged_policy_list_response_item import TaggedPolicyListResponseItem
from .tagged_project_list_response_item import TaggedProjectListResponseItem
from .team import Team
from .team_self_response import TeamSelfResponse
from .test_smtp_publisher_config_body import TestSmtpPublisherConfigBody
from .upload_bom_body import UploadBomBody
from .upload_vex_1_body import UploadVex1Body
from .user_principal import UserPrincipal
from .validate_credentials_body import ValidateCredentialsBody
from .validate_oidc_access_token_body import ValidateOidcAccessTokenBody
from .vex_submit_request import VexSubmitRequest
from .violation_analysis import ViolationAnalysis
from .violation_analysis_analysis_state import ViolationAnalysisAnalysisState
from .violation_analysis_comment import ViolationAnalysisComment
from .violation_analysis_request import ViolationAnalysisRequest
from .violation_analysis_request_analysis_state import (
    ViolationAnalysisRequestAnalysisState,
)
from .violation_analysis_violation_analysis_state import (
    ViolationAnalysisViolationAnalysisState,
)
from .visible_teams import VisibleTeams
from .vulnerability import Vulnerability
from .vulnerability_alias import VulnerabilityAlias
from .vulnerability_metrics import VulnerabilityMetrics
from .vulnerability_severity import VulnerabilitySeverity

__all__ = (
    "About",
    "AboutProviderData",
    "AboutProviderDataAdditionalProperty",
    "AclMappingRequest",
    "AffectedComponent",
    "AffectedComponentIdentityType",
    "AffectedComponentVersionType",
    "AffectedVersionAttribution",
    "AffectedVersionAttributionSource",
    "Analysis",
    "AnalysisAnalysisJustification",
    "AnalysisAnalysisResponse",
    "AnalysisAnalysisState",
    "AnalysisComment",
    "AnalysisRequest",
    "AnalysisRequestAnalysisJustification",
    "AnalysisRequestAnalysisResponse",
    "AnalysisRequestAnalysisState",
    "ApiKey",
    "BomSubmitRequest",
    "BomUploadResponse",
    "CloneProjectRequest",
    "Component",
    "ComponentClassifier",
    "ComponentProperty",
    "ComponentPropertyPropertyType",
    "ConfigProperty",
    "ConfigPropertyPropertyType",
    "CreateScheduledNotificationRuleRequest",
    "CreateScheduledNotificationRuleRequestNotificationLevel",
    "CreateScheduledNotificationRuleRequestScope",
    "Cwe",
    "DataClassification",
    "DataClassificationDirection",
    "DependencyGraphResponse",
    "DependencyMetrics",
    "ExternalReference",
    "ExternalReferenceType",
    "Finding",
    "FindingAnalysis",
    "FindingAnalysisState",
    "FindingAttrib",
    "FindingAttribAnalyzerIdentity",
    "FindingComponent",
    "FindingVulnerability",
    "ForceChangePasswordBody",
    "Framework",
    "GetAllComponentsSortOrder",
    "GetAllNotificationRulesSortOrder",
    "GetAllNotificationRulesTriggerType",
    "GetAllPermissionsResponse200",
    "GetAllServicesSortOrder",
    "GetAllTagsSortOrder",
    "GetAllVulnerabilitiesSortOrder",
    "GetChildrenProjectsByClassifierSortOrder",
    "GetChildrenProjectsByTagSortOrder",
    "GetChildrenProjectsSortOrder",
    "GetComponentByHashSortOrder",
    "GetComponentByIdentitySortOrder",
    "GetCwesSortOrder",
    "GetDependencyGraphForComponentResponse200",
    "GetFindingsByProjectSource",
    "GetLicenseGroupsSortOrder",
    "GetLicensesSortOrder",
    "GetPoliciesSortOrder",
    "GetProjectsByClassifierSortOrder",
    "GetProjectsByTagSortOrder",
    "GetProjectsSortOrder",
    "GetProjectsWithoutDescendantsOfSortOrder",
    "GetRepositoriesByTypeSortOrder",
    "GetRepositoriesByTypeType",
    "GetRepositoriesSortOrder",
    "GetTaggedCollectionProjectsSortOrder",
    "GetTaggedNotificationRulesSortOrder",
    "GetTaggedPoliciesSortOrder",
    "GetTaggedProjectsSortOrder",
    "GetTagsForPolicySortOrder",
    "GetTagsSortOrder",
    "GetViolationsByComponentSortOrder",
    "GetViolationsByProjectSortOrder",
    "GetViolationsSortOrder",
    "GetVulnerabilitiesByComponentSortOrder",
    "IdentifiableObject",
    "InvalidBomProblemDetails",
    "IsTokenBeingProcessedResponse",
    "LdapUser",
    "License",
    "LicenseGroup",
    "ManagedUser",
    "MappedLdapGroup",
    "MappedLdapGroupRequest",
    "MappedOidcGroup",
    "MappedOidcGroupRequest",
    "NotificationPublisher",
    "NotificationRule",
    "NotificationRuleNotificationLevel",
    "NotificationRuleNotifyOnItem",
    "NotificationRuleScope",
    "NotificationRuleTriggerType",
    "OidcGroup",
    "OidcUser",
    "OrganizationalContact",
    "OrganizationalEntity",
    "Permission",
    "Policy",
    "PolicyCondition",
    "PolicyConditionOperator",
    "PolicyConditionSubject",
    "PolicyOperator",
    "PolicyViolation",
    "PolicyViolationState",
    "PolicyViolationType",
    "PortfolioMetrics",
    "ProblemDetails",
    "Project",
    "ProjectClassifier",
    "ProjectCollectionLogic",
    "ProjectMetadata",
    "ProjectMetrics",
    "ProjectMetricsCollectionLogic",
    "ProjectOperationProblemDetails",
    "ProjectOperationProblemDetailsErrors",
    "ProjectProperty",
    "ProjectPropertyPropertyType",
    "ProjectVersion",
    "Publisher",
    "Repository",
    "RepositoryMetaComponent",
    "RepositoryMetaComponentRepositoryType",
    "RepositoryType",
    "RetrieveProjectsSortOrder",
    "Score",
    "ScoreBusinessImpact",
    "ScoreLikelihood",
    "ScoreTechnicalImpact",
    "SearchResult",
    "SearchResultResults",
    "SearchResultResultsAdditionalPropertyItem",
    "ServiceComponent",
    "Tag",
    "TaggedCollectionProjectListResponseItem",
    "TaggedPolicyListResponseItem",
    "TaggedProjectListResponseItem",
    "TagListResponseItem",
    "TagOperationProblemDetails",
    "TagOperationProblemDetailsErrors",
    "Team",
    "TeamSelfResponse",
    "TestSmtpPublisherConfigBody",
    "UploadBomBody",
    "UploadVex1Body",
    "UserPrincipal",
    "ValidateCredentialsBody",
    "ValidateOidcAccessTokenBody",
    "VexSubmitRequest",
    "ViolationAnalysis",
    "ViolationAnalysisAnalysisState",
    "ViolationAnalysisComment",
    "ViolationAnalysisRequest",
    "ViolationAnalysisRequestAnalysisState",
    "ViolationAnalysisViolationAnalysisState",
    "VisibleTeams",
    "Vulnerability",
    "VulnerabilityAlias",
    "VulnerabilityMetrics",
    "VulnerabilitySeverity",
)



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.create_scheduled_notification_rule_request_notification_level import (
    CreateScheduledNotificationRuleRequestNotificationLevel,
)
from ..models.create_scheduled_notification_rule_request_scope import (
    CreateScheduledNotificationRuleRequestScope,
)

if TYPE_CHECKING:
    from ..models.publisher import Publisher


T = TypeVar("T", bound="CreateScheduledNotificationRuleRequest")


@_attrs_define
class CreateScheduledNotificationRuleRequest:
    """
    Attributes:
        name (str):
        scope (CreateScheduledNotificationRuleRequestScope):
        notification_level (CreateScheduledNotificationRuleRequestNotificationLevel):
        publisher (Publisher):
    """

    name: str
    scope: CreateScheduledNotificationRuleRequestScope
    notification_level: CreateScheduledNotificationRuleRequestNotificationLevel
    publisher: "Publisher"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        scope = self.scope.value

        notification_level = self.notification_level.value

        publisher = self.publisher.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "scope": scope,
                "notificationLevel": notification_level,
                "publisher": publisher,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.publisher import Publisher

        d = dict(src_dict)
        name = d.pop("name")

        scope = CreateScheduledNotificationRuleRequestScope(d.pop("scope"))

        notification_level = CreateScheduledNotificationRuleRequestNotificationLevel(
            d.pop("notificationLevel")
        )

        publisher = Publisher.from_dict(d.pop("publisher"))

        create_scheduled_notification_rule_request = cls(
            name=name,
            scope=scope,
            notification_level=notification_level,
            publisher=publisher,
        )

        create_scheduled_notification_rule_request.additional_properties = d
        return create_scheduled_notification_rule_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
